<?php

App::uses('BuzzVouchersAppModel', 'BuzzVouchers.Model');
App::uses('CustomerAddress', 'BuzzCustomers.Model');
App::uses('GiftVoucherPurchaseState', 'BuzzVouchers.Model');

class GiftVoucherPurchase extends BuzzVouchersAppModel {

/**
 * Belongs to associations
 *
 * @var array
 */
	public $belongsTo = array(
		'CustomerAddress' => array(
			'className' => 'BuzzCustomers.CustomerAddress'
		),
		'DeliveryAddress' => array(
			'className' => 'BuzzCustomers.CustomerAddress',
			'foreignKey' => 'delivery_address_id'
		),
		'GiftVoucherPurchaseState' => array(
			'className' => 'BuzzVouchers.GiftVoucherPurchaseState'
		)
	);

/**
 * Has many associations
 *
 * @var array
 */
	public $hasMany = array(
		'GiftVoucherPurchaseItem' => array(
			'className' => 'BuzzVouchers.GiftVoucherPurchaseItem'
		)
	);

/**
 * Basket ID
 *
 * @var int
 */
	protected $_basketId = null;

/**
 * Read for view
 *
 * @param int $id Booking ID
 * @param array $params
 * @return array
 */
	public function readForView($id, $params = []) {
		$defaults = array(
			'contain' => array(
				'CustomerAddress' => array(
					'Country',
					'UsState'
				),
				'DeliveryAddress' => array(
					'Country',
					'UsState'
				),
				'GiftVoucherPurchaseItem' => array(
					'GiftVoucher'
				)
			)
		);
		$params = Hash::merge($defaults, $params);

		$data = parent::readForView($id, $params);

		if (!empty($data['GiftVoucherPurchaseItem'])) {
			$this->VoucherActivity = $this->GiftVoucherPurchaseItem->GiftVoucher->VoucherActivity;
			$activities = $this->VoucherActivity->find(
				'all',
				array(
					'conditions' => array(
						'VoucherActivity.id' => Hash::extract($data, 'GiftVoucherPurchaseItem.{n}.GiftVoucher.voucher_activity_id')
					)
				)
			);
			$activities = Hash::combine($activities, '{n}.VoucherActivity.id', '{n}.VoucherActivity');
			foreach ($data['GiftVoucherPurchaseItem'] as &$value) {
				$activityId = $value['GiftVoucher']['voucher_activity_id'];
				$value['GiftVoucher']['VoucherActivity'] = $activities[$activityId];
			}
		}

		return $data;
	}

/**
 * Get the current basket ID from the session
 *
 * @return int
 */
	public function getBasketId() {
		if (empty($this->_basketId)) {
			$this->_basketId = CakeSession::read('GiftVoucherPurchase.id');
		}
		return $this->_basketId;
	}

/**
 * Clear the basket ID from the session
 *
 * @return bool
 */
	public function clearBasketId() {
		$this->_basketId = null;
		CakeSession::delete('GiftVoucherPurchase.id');
		return true;
	}

/**
 * Returns a hashed basket ID
 *
 * @param int $id (Optional) basket ID
 * @return string
 */
	public function hashBasketId($id = null) {
		$id = $id ?: $this->getBasketId();
		return md5($id);
	}

/**
 * Get the current basket
 *
 * @return array
 */
	public function getBasket() {
		$basketId = $this->getBasketId();
		if (empty($basketId)) {
			return [];
		}
		$params = array(
			'conditions' => array(
				'GiftVoucherPurchase.gift_voucher_purchase_state_id' => GiftVoucherPurchaseState::UNPAID
			)
		);
		return $this->readForView($basketId, $params);
	}

/**
 * Get a gift voucher purchase to display to the customer.
 *
 * @param int $id Purchase ID
 * @return array
 */
	public function getPurchase($id) {
		$params = array(
			'contain' => array(
				'GiftVoucherPurchaseItem' => array(
					'GiftVoucher',
					// Need to make sure we retrieve the purchased vouchers.
					'GiftVoucherPurchaseItemVoucher'
				)
			),
			'conditions' => array(
				'GiftVoucherPurchase.gift_voucher_purchase_state_id' => array(
					GiftVoucherPurchaseState::COMPLETE,
					GiftVoucherPurchaseState::API_FAILED
				)
			)
		);
		return $this->readForView($id, $params);
	}

/**
 * Adds items to the basket
 *
 * @param array $items
 * @return bool
 */
	public function addItems(array $items) {
		$basketId = $this->getBasketId();

		// Make sure we don't save items with zero quantity to the database.
		$removeItems = [];
		foreach ($items as $key => $val) {
			if ((int)$val['quantity'] === 0) {
				unset($items[$key]);
			}
			// Flag existing items that are getting removed.
			if (!empty($val['id'])) {
				$removeItems[] = $val['id'];
			}
		}

		// Remove unwanted items.
		if (!empty($removeItems)) {
			$this->GiftVoucherPurchaseItem->deleteAll(['GiftVoucherPurchaseItem.id' => $removeItems]);
			unset($removeItems);
		}

		// Get the details of the gift vouchers being purchased so that we can
		// calculate costs.
		$vouchers = $this->GiftVoucherPurchaseItem->GiftVoucher->find(
			'all',
			array(
				'conditions' => array(
					'GiftVoucher.id' => Hash::extract($items, '{n}.gift_voucher_id')
				)
			)
		);
		$vouchers = Hash::combine($vouchers, '{n}.GiftVoucher.id', '{n}.GiftVoucher');

		foreach ($items as $key => &$item) {

			$giftVoucherId = $item['gift_voucher_id'];

			if (!empty($vouchers[$giftVoucherId])) {
				$item['item_unit_cost'] = $vouchers[$giftVoucherId]['price'];
				$item['total'] = $item['quantity'] * $vouchers[$giftVoucherId]['price'];
			} else {
				unset($items[$key]);
			}

		}

		if ($basketId === null) {
			// Create a new basket.
			$data = array(
				'GiftVoucherPurchase' => array(
					'id' => $basketId,
					'gift_voucher_purchase_state_id' => GiftVoucherPurchaseState::UNPAID
				),
				'GiftVoucherPurchaseItem' => $items
			);

			$result = $this->saveAssociated($data) !== false;
			$basketId = $result === true ? $this->id : null;

		} else {
			// Make sure we save to the current basket.
			$items = Hash::insert($items, '{n}.gift_voucher_purchase_id', $basketId);
			$result = $this->GiftVoucherPurchaseItem->saveMany($items) !== false;

		}

		// Update the basket total and session.
		if ($result === true) {
			CakeSession::write('GiftVoucherPurchase.id', $basketId);
			$this->_updateBasketTotal();
		} else {
			$this->clearBasketId();
		}

		return $result;
	}

/**
 * Update the current basket total.
 *
 * @return bool
 */
	protected function _updateBasketTotal() {
		$basketId = $this->getBasketId();
		// Get all items in the basket.
		$basketItems = $this->GiftVoucherPurchaseItem->find(
			'all',
			array(
				'conditions' => array(
					'GiftVoucherPurchaseItem.gift_voucher_purchase_id' => $basketId
				)
			)
		);

		// Calculate the subtotal.
		$subtotal = array_sum(Hash::extract($basketItems, '{n}.GiftVoucherPurchaseItem.total'));

		// Calculate the delivery cost.
		$this->id = $basketId;
		if ((bool)$this->field('evouchers') === false) {
			$totalItems = array_sum(
				Hash::extract(
					$basketItems,
					'{n}.GiftVoucherPurchaseItem.quantity'
				)
			);
			$deliveryCost = $totalItems * Configure::read('BuzzVouchers.postage_cost');
		} else {
			$deliveryCost = 0;
		}

		// Save the update total.
		$data = array(
			'id' => $basketId,
			'subtotal' => $subtotal,
			'delivery_cost' => $deliveryCost,
			'total_cost' => $subtotal + $deliveryCost
		);
		return $this->save($data) !== false;
	}

/**
 * Add the customer delivery details to the order
 *
 * @param bool $isEvoucher
 * @param array $address
 * @param string $giftMessage
 * @return bool
 */
	public function addDeliveryDetails($isEvoucher, array $address, $giftMessage = null) {
		$data = array(
			'GiftVoucherPurchase' => array(
				'id' => $this->getBasketId(),
				'evouchers' => $isEvoucher,
			)
		);
		if ($isEvoucher === true) {
			$data['GiftVoucherPurchase']['gift_message'] = $giftMessage;
		} else {
			$data['DeliveryAddress'] = $address;
		}
		$result = $this->saveAssociated($data) !== false;
		$this->_updateBasketTotal();
		return $result;
	}

/**
 * Add the customer billing details to the order
 *
 * @param array $customer
 * @return bool
 */
	public function addBillingDetails(array $customer) {
		$data = array(
			'GiftVoucherPurchase' => array(
				'id' => $this->getBasketId()
			),
			'CustomerAddress' => $customer
		);
		return $this->saveAssociated($data) !== false;
	}

/**
 * Complete the purchase
 *
 * @param bool $paid True if payment successfully made
 * @return bool
 */
	public function completePurchase($paid) {
		$id = $this->getBasketId();

		$basket = $this->getBasket($id);

		$response = true;

		if ((int)$basket['GiftVoucherPurchase']['gift_voucher_purchase_state_id'] === GiftVoucherPurchaseState::UNPAID) {
			if ($paid === true) {
				// Create the vouchers
				$result = $this->_createVouchers($basket);

				$data = array(
					'id' => $id,
					'gift_voucher_purchase_state_id' => $result === true ? GiftVoucherPurchaseState::COMPLETE : GiftVoucherPurchaseState::API_FAILED,
					'completed_date' => gmdate('Y-m-d H:i:s')
				);

				$response = $this->save($data);

			} else {
				// Mark as payment failed.
				$data = array(
					'id' => $id,
					'gift_voucher_purchase_state_id' => GiftVoucherPurchaseState::PAYMENT_FAILED
				);
				$response = $this->save($data);

			}

			// Clear the basket session.
			$this->clearBasketId();

		}

		return $response;
	}

/**
 * Create vouchers
 *
 * @param array $basket
 * @return bool
 */
	protected function _createVouchers(array $basket) {
		$data = [];
		$response = true;

		foreach ($basket['GiftVoucherPurchaseItem'] as $item) {

			$result = ClassRegistry::init('BuzzVouchers.VoucherApi')->createVoucher(
				$basket['CustomerAddress']['first_name'],
				$basket['CustomerAddress']['last_name'],
				$basket['CustomerAddress']['email'],
				$basket['CustomerAddress']['telephone'],
				CustomerAddress::generateAddressArray(
					// For postal delivery we want to pass the delivery address
					// to the API; for evouchers the customer's address.
					$basket['GiftVoucherPurchase']['evouchers'] === true ? $basket['CustomerAddress'] : $basket['DeliveryAddress']
				),
				$basket['GiftVoucherPurchase']['evouchers'] === false ? $basket['GiftVoucherPurchase']['gift_message'] : null,
				$item['GiftVoucher']['api_reference'],
				$item['quantity'],
				$basket['GiftVoucherPurchase']['evouchers'],
				null,
				[
					'vendorTxCode' => '',
					'vpstxId' => '',
					'status' => 'OK',
					'txauthno' => ''
				],
				$item['total']
			);

			if ($result !== false) {
				foreach ($result['Voucher'] as $voucher) {
					$data[] = array(
						'gift_voucher_purchase_item_id' => $item['id'],
						'description' => $voucher['description'],
						'code' => $voucher['code'],
						'expiry_date' => $voucher['expiry_date']
					);
				}
			} else {
				$response = false;
			}

		}

		if (!empty($data)) {
			$this->GiftVoucherPurchaseItem->GiftVoucherPurchaseItemVoucher->saveMany($data);
		}

		return $response;
	}

}
