<?php

App::uses('BuzzSourceAppModel', 'BuzzSource.Model');

class VoucherApi extends BuzzSourceAppModel {

/**
 * Create a gift voucher.
 *
 * @param string $firstName
 * @param string $lastName
 * @param string $email
 * @param string $telephone
 * @param array $address Address generated by CustomerAddress::generateAddressArray()
 * @param string $message Gift message
 * @param string $templateId Voucher template
 * @param int $quantity Quantity of vouchers being purchased
 * @param bool $isEvoucher True if an e-voucher
 * @param string $ip IP address
 * @param array $payment Payment details for the API
 * @param float $total Amount paid
 * @return array|bool
 */
	public function createVoucher($firstName, $lastName, $email, $telephone, $address, $message, $templateId, $quantity, $isEvoucher, $ip, array $payment, $total = 0) {
		$result = $this->find('all', array(
			'method' => 'CreateVoucher',
			'conditions' => array(
				'firstname' => urlencode($firstName),
				'lastname' => urlencode($lastName),
				'email' => urlencode($email),
				'telephone' => urlencode($telephone),
				'addressline1' => urlencode($address['address_line_1']),
				'addressline2' => urlencode($address['address_line_2']),
				'addressline3' => urlencode($address['address_line_3']),
				'addressline4' => urlencode($address['address_line_4']),
				'postcode' => urlencode($address['postcode']),
				'message' => urlencode($message),
				'templateId' => urlencode($templateId),
				'quantity' => (int)$quantity,
				'isevoucher' => $isEvoucher === true ? 'true' : 'false',
				'ipaddress' => $ip === null ? $_SERVER['REMOTE_ADDR'] : $ip,
				'vendortxcode' => $payment['vendorTxCode'],
				'vpstxId' => $payment['vpstxId'],
				'status' => $payment['status'],
				'txauthno' => $payment['txauthno'],
				'totalPaid' => $total
			)
		));

		return !empty($result['VoucherApi']['d']) ? $this->_processVouchers($result['VoucherApi']['d']) : false;
	}

/**
 * Process gift vouchers data from the API
 *
 * @param array $data Response from API
 * @return array
 */
	protected function _processVouchers($data) {
		$result = [];

		foreach ($data as $voucher) {
			$result['Voucher'][] = array(
				'id' => $voucher['Id'],
				'description' => $voucher['Description'],
				'code' => $voucher['VoucherNo'],
				'expiry_date' => $voucher['ExpiryDate']
			);
		}

		return $result;
	}

}
